/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.tooling.java2wsdl;

import java.util.HashMap;
import java.util.Map;

import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Types;
import org.ow2.easywsdl.wsdl.api.WSDLException;

/**
 * @author Gael Blondelle - EBM WebSourcing
 * @author Mathias Beldame - EBM WebSourcing
 */
public class WSDLGenerationContext {

	private Description description;

	private Map<String, Schema> schemas = new HashMap<String, Schema>();


	public Description processWSDL() throws WSDLException {
		
		Types types = description.createTypes();
		description.setTypes(types);
		
		for (Map.Entry<String, Schema> schemaEntry : schemas.entrySet()) {
			types.addSchema(schemaEntry.getValue());
		}
				
		return description;
	}

	public Description getDescription() {
		return description;
	}

	public void setDescription(Description description) {
		this.description = description;
	}

	public Schema getSchema(String namespace) {
		Schema schema = schemas.get(namespace);

		if (schema == null) {
			try {
				schema = SchemaFactory.newInstance().newSchema();
				schema.setTargetNamespace(namespace);
			} catch (SchemaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			schemas.put(namespace, schema);
		}

		return schema;
	}
}
