/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.tooling.java2wsdl.util;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

/**
 * @author Mathias Beldame - EBM WebSourcing
 */
public class XMLSorter {

		/**
	     * Parses the xml String and return it with nodes in the order
	     * following the ws i recommendations (types at the beginning)
	     * 
	     * @param xmlDocument
	     *            the xml document to sort. Must be non null
	     * @param encoding
	     *            the encoding to use
	     *   
	     * @return pretty printed string if no error occurs. If an error occurs,
	     *               return an empty String
	     */
	    public static String sort(final Document xmlDocument,String encoding) {
	        String result = "";
	        try {
	            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
	            XMLSorter.sort(xmlDocument, outStream,encoding);
	            result = outStream.toString(encoding);
	        } catch (final Exception e) {
	            System.err.println("write_dom failed:" + e);
	            // if an error occurs, the result will be the original string
	        }
	        return result;

	    }

	    /**
	     * parse the xml Document and return it with its nodes sorted
	     * Use the encoding defined at the parsing or in the document 
	     * (utf8 is used if no encoding is defined)
	     * @param xmlDocument
	     *            the xml document to pretty print. Must be non null
	     * @return pretty printed string if no error occurs. If an error occurs,
	     *               return an empty String
	     */
	    public static String sort(final Document xmlDocument) {
	    	
	        return sort(xmlDocument,getEncoding(xmlDocument));
	    }

	    /**
	     * parse the xml Document and return it with its nodes sorted
	     * Use the encoding defined at the parsing or in the document 
	     * (utf8 is used if no encoding is defined)
	     * @param xml
	     *            the xml document to pretty print. Must be non null
	     * @return pretty printed string if no error occurs. If an error occurs,
	     *               return an empty String
	     */
	    public static Document sortNodes( final Document xml) {
	    	
	    	String tempXML = XMLSorter.sort(xml);
	    	
	    	try {
	    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware(true);
	    	DocumentBuilder builder;
			
				builder = factory.newDocumentBuilder();
			
	        Document doc2 = builder.parse(new InputSource(new StringReader(tempXML)));
	    	
	        return doc2;
	        
	    	} catch (Exception e) {
				e.printStackTrace();
			}
	    	
	    	return xml; 
	    }
	    
	    /**
	     * Sort the node into the output stream.
	     *
	     * @param node
	     * @param out
	     * @throws Exception
	     */
	    public static void sort(Node node, OutputStream out,String encoding) throws Exception {
	        Source source = new DOMSource(node);
	        Source stylesheetSource = getStyleSheetSource();
	        Transformer transformer = XMLPrettyPrinter.getTransformer(stylesheetSource);
	        transformer.setOutputProperty(OutputKeys.ENCODING,encoding);
	        transformer.transform(source, new StreamResult(out));
	    }

	    /**
	     * Return the encoding of the document.
	     * @param xmlDocument
	     * @return InputEncoding or the XmlEncoding of the document, UTF-8 if not found
	     */
	    public static String getEncoding(Document xmlDocument){
	    	String encoding = xmlDocument.getInputEncoding();
	    	if(encoding == null){
	    		encoding = xmlDocument.getXmlEncoding();
	    	}
	    	if(encoding == null){
	    		encoding = "UTF-8";
	    	}
	    	return encoding;
	    }
	    
	    private static Source getStyleSheetSource() {
	        return new StreamSource(XMLSorter.class.getResourceAsStream("/test/classify.xsl"));
	    }
	
}
